package com.me.brickbuster.entity;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.me.brickbuster.entity.powerup.PowerUpType;
import com.me.brickbuster.physics.PhysicsBody;
import com.me.brickbuster.state.PlayState;

public class Brick extends Entity implements PhysicsBody {

	public static final Color DEFAULT_COLOR = Color.FOREST;
	public static final float BRICK_WIDTH = 5f;
	public static final float BRICK_HEIGHT = 2.5f;

	private PowerUpType powerUpType;
	private Color color;

	private Vector2[] vertices;

	private Body body;

	public Brick(PlayState state, PowerUpType powerUpType, float x, float y) {
		this(state, powerUpType, true, x, y);
	}

	public Brick(PlayState state, PowerUpType powerUpType, boolean hidePowerup, float x, float y) {
		this(state, powerUpType, DEFAULT_COLOR, hidePowerup, x, y);
	}

	public Brick(PlayState state, PowerUpType powerUpType, Color color, boolean hidePowerUp, float x, float y) {
		super(state, x, y);
		this.powerUpType = powerUpType;
		this.color = powerUpType != null && !hidePowerUp? powerUpType.getColor() : color;
		this.vertices = new Vector2[] {
			new Vector2(x, y),
			new Vector2(x + BRICK_WIDTH, y),
			new Vector2(x + BRICK_WIDTH, y + BRICK_HEIGHT),
			new Vector2(x, y + BRICK_HEIGHT)
		};

		BodyDef brickBody = new BodyDef();
		brickBody.type = BodyDef.BodyType.StaticBody;
		brickBody.position.set(pos.cpy());

		PolygonShape brickShape = new PolygonShape();
		brickShape.setAsBox(BRICK_WIDTH/2, BRICK_HEIGHT/2,
				new Vector2(BRICK_WIDTH/2,BRICK_HEIGHT/2), 0);

		FixtureDef brickFixture = new FixtureDef();
		brickFixture.shape = brickShape;
		brickFixture.friction = 0f;

		body = state.world.createBody(brickBody);
		body.createFixture(brickFixture);
		body.setUserData(this);

		brickShape.dispose();
	}

	@Override
	public void render(ShapeRenderer sr) {
		sr.begin(ShapeType.Filled);
		sr.setColor(color);
		sr.rect(getX()*PlayState.PIXEL_PER_METER,
				getY()*PlayState.PIXEL_PER_METER,
				BRICK_WIDTH*PlayState.PIXEL_PER_METER,
				BRICK_HEIGHT*PlayState.PIXEL_PER_METER);
		sr.end();
	}

	@Override
	public void update(float dt) {
	}

	@Override
	public Body getBody() {
		return body;
	}

	public boolean hit() {
		if (state.bricks.size()-1 <= Ball.BLOCKS_FOR_BOOST) {
			for (Ball ball : state.balls) {
				ball.setSpeed(Ball.BOOST_SPEED);
			}
		}

		if (powerUpType != null) {
			state.powerUps.add(powerUpType.createInstance(state, this));
		}
		return true;
	}

	public Vector2[] getVertices() {
		return vertices;
	}

}
