package com.me.brickbuster.entity;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.EdgeShape;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.me.brickbuster.physics.PhysicsBody;
import com.me.brickbuster.state.PlayState;
import net.dermetfan.utils.Pair;

public class Paddle extends Entity implements PhysicsBody {

	public static final Color STICKY_COLOR = Color.GRAY;
	public static final Color PADDLE_COLOR = Color.BLACK;

	public static final float DEFAULT_WIDTH = 7.5f;
	public static final float PADDLE_HEIGHT = 0.75f;
	public static final float PADDLE_Y = 1.25f;
	public static final float PADDLE_SPEED = 40f;

	private float width = DEFAULT_WIDTH;
	private boolean sticky = false;

	private Body body;

	public Paddle(PlayState brickBuster) {
		super(brickBuster, PlayState.BOARD_WIDTH/2, PADDLE_Y);

		BodyDef paddleBody = new BodyDef();
		paddleBody.type = BodyDef.BodyType.KinematicBody;
		paddleBody.position.set(pos);

		EdgeShape paddleShape = new EdgeShape();
		paddleShape.set(new Vector2(-DEFAULT_WIDTH/2,PADDLE_HEIGHT),
				new Vector2(DEFAULT_WIDTH/2, PADDLE_HEIGHT));

		FixtureDef paddleFixture = new FixtureDef();
		paddleFixture.shape = paddleShape;
		paddleFixture.isSensor = true;

		body = state.world.createBody(paddleBody);
		body.createFixture(paddleFixture);
		body.setUserData(this);

		paddleShape.dispose();
	}

	@Override
	public void render(ShapeRenderer sr) {
		sr.begin(ShapeType.Filled);
		sr.setColor(sticky? STICKY_COLOR : PADDLE_COLOR);
		sr.rect((getX() - width/2) * PlayState.PIXEL_PER_METER,
				getY() * PlayState.PIXEL_PER_METER,
				width * PlayState.PIXEL_PER_METER,
				PADDLE_HEIGHT * PlayState.PIXEL_PER_METER);
		sr.end();
	}

	@Override
	public void update(float dt) {
		if (Gdx.input.isKeyPressed(Input.Keys.LEFT)) {
			if (pos.x - width/2 - PADDLE_SPEED * dt < 0) {
				setX(width/2);
				return;
			}
			pos.x = pos.x - PADDLE_SPEED * dt;

			for (Ball ball : state.balls) {
				if (ball.isStuck()) {
					ball.setX(ball.getX() - PADDLE_SPEED * dt);
				}
			}
		}
		if (Gdx.input.isKeyPressed(Input.Keys.RIGHT)) {
			if (pos.x + width/2 + PADDLE_SPEED * dt > PlayState.BOARD_WIDTH) {
				setX(PlayState.BOARD_WIDTH - width/2);
				return;
			}
			pos.x = pos.x + PADDLE_SPEED * dt;

			for (Ball ball : state.balls) {
				if (ball.isStuck()) {
					ball.setX(ball.getX() + PADDLE_SPEED * dt);
				}
			}
		}
	}

	@Override
	public Body getBody() {
		return body;
	}

	public Pair<Vector2, Vector2> getTopEdge() {
		return new Pair<Vector2, Vector2>(
				new Vector2(pos.x - width/2, pos.y + PADDLE_HEIGHT),
				new Vector2(pos.x + width/2, pos.y + PADDLE_HEIGHT)
		);
	}

	public float getWidth() {
		return width;
	}

	public void setWidth(float width) {
		this.width = width;
	}

	public boolean isSticky() {
		return sticky;
	}

	public void setSticky(boolean sticky) {
		this.sticky = sticky;
	}




}
